@@ -1,18 +1,25 @@
 # coding=utf-8
 """Sopel Plugins Command Line Interface (CLI): ``sopel-plugins``"""
-from __future__ import unicode_literals, absolute_import, print_function, division
+from __future__ import absolute_import, division, print_function, unicode_literals
 
 import argparse
 import inspect
 import operator
 
-from sopel import plugins, tools
-
+from sopel import config, plugins, tools
 from . import utils
 
 
+ERR_CODE = 1
+"""Error code: program exited with an error"""
+
+
 def build_parser():
-    """Configure an argument parser for ``sopel-plugins``"""
+    """Configure an argument parser for ``sopel-plugins``.
+
+    :return: the argument parser
+    :rtype: :class:`argparse.ArgumentParser`
+    """
     parser = argparse.ArgumentParser(
         description='Sopel plugins tool')
 
@@ -150,7 +157,12 @@ def build_parser():
 
 
 def handle_list(options):
-    """List Sopel plugins"""
+    """List Sopel plugins.
+
+    :param options: parsed arguments
+    :type options: :class:`argparse.Namespace`
+    :return: 0 if everything went fine
+    """
     settings = utils.load_settings(options)
     no_color = options.no_color
     name_only = options.name_only
@@ -227,7 +239,13 @@ def handle_list(options):
 
 
 def handle_show(options):
-    """Show plugin details"""
+    """Show plugin details.
+
+    :param options: parsed arguments
+    :type options: :class:`argparse.Namespace`
+    :return: 0 if everything went fine;
+             1 if the plugin doesn't exist or can't be loaded
+    """
     plugin_name = options.name
     settings = utils.load_settings(options)
     usable_plugins = plugins.get_usable_plugins(settings)
@@ -235,7 +253,7 @@ def handle_show(options):
     # plugin does not exist
     if plugin_name not in usable_plugins:
         tools.stderr('No plugin named %s' % plugin_name)
-        return 1
+        return ERR_CODE
 
     plugin, is_enabled = usable_plugins[plugin_name]
     description = {
@@ -267,7 +285,7 @@ def handle_show(options):
 
     if not loaded:
         print('Loading failed')
-        return 1
+        return ERR_CODE
 
     print('Loaded successfully')
     print('Setup:', 'yes' if plugin.has_setup() else 'no')
@@ -277,7 +295,13 @@ def handle_show(options):
 
 
 def handle_configure(options):
-    """Configure a Sopel plugin with a config wizard"""
+    """Configure a Sopel plugin with a config wizard.
+
+    :param options: parsed arguments
+    :type options: :class:`argparse.Namespace`
+    :return: 0 if everything went fine;
+             1 if the plugin doesn't exist or can't be loaded
+    """
     plugin_name = options.name
     settings = utils.load_settings(options)
     usable_plugins = plugins.get_usable_plugins(settings)
@@ -285,21 +309,27 @@ def handle_configure(options):
     # plugin does not exist
     if plugin_name not in usable_plugins:
         tools.stderr('No plugin named %s' % plugin_name)
-        return 1
+        return ERR_CODE
 
     plugin, is_enabled = usable_plugins[plugin_name]
     try:
         plugin.load()
     except Exception as error:
         tools.stderr('Cannot load plugin %s: %s' % (plugin_name, error))
-        return 1
+        return ERR_CODE
 
     if not plugin.has_configure():
         tools.stderr('Nothing to configure for plugin %s' % plugin_name)
         return 0  # nothing to configure is not exactly an error case
 
     print('Configure %s' % plugin.get_label())
-    plugin.configure(settings)
+    try:
+        plugin.configure(settings)
+    except KeyboardInterrupt:
+        tools.stderr(
+            '\nOperation cancelled; the config file has not been modified.')
+        return ERR_CODE  # cancelled operation
+
     settings.save()
 
     if not is_enabled:
@@ -340,6 +370,10 @@ def _handle_disable_plugin(settings, plugin_name, force):
 
 
 def display_unknown_plugins(unknown_plugins):
+    """Print an error message when one or more plugins are unknown.
+
+    :param list unknown_plugins: list of unknown plugins
+    """
     # at least one of the plugins does not exist
     tools.stderr(utils.get_many_text(
         unknown_plugins,
@@ -350,7 +384,14 @@ def display_unknown_plugins(unknown_plugins):
 
 
 def handle_disable(options):
-    """Disable Sopel plugins"""
+    """Disable Sopel plugins.
+
+    :param options: parsed arguments
+    :type options: :class:`argparse.Namespace`
+    :return: 0 if everything went fine;
+             1 if the plugin doesn't exist,
+             or if attempting to disable coretasks (required)
+    """
     plugin_names = options.names
     force = options.force
     ensure_remove = options.remove
@@ -361,7 +402,7 @@ def handle_disable(options):
     # coretasks is sacred
     if 'coretasks' in plugin_names:
         tools.stderr('Plugin coretasks cannot be disabled.')
-        return 1  # do nothing and return an error code
+        return ERR_CODE  # do nothing and return an error code
 
     unknown_plugins = [
         name
@@ -370,7 +411,7 @@ def handle_disable(options):
     ]
     if unknown_plugins:
         display_unknown_plugins(unknown_plugins)
-        return 1  # do nothing and return an error code
+        return ERR_CODE  # do nothing and return an error code
 
     # remove from enabled if asked
     if ensure_remove:
@@ -451,7 +492,13 @@ def _handle_enable_plugin(settings, usable_plugins, plugin_name, allow_only):
 
 
 def handle_enable(options):
-    """Enable a Sopel plugin"""
+    """Enable a Sopel plugin.
+
+    :param options: parsed arguments
+    :type options: :class:`argparse.Namespace`
+    :return: 0 if everything went fine;
+             1 if the plugin doesn't exist
+    """
     plugin_names = options.names
     allow_only = options.allow_only
     settings = utils.load_settings(options)
@@ -465,7 +512,7 @@ def handle_enable(options):
     ]
     if unknown_plugins:
         display_unknown_plugins(unknown_plugins)
-        return 1  # do nothing and return an error code
+        return ERR_CODE  # do nothing and return an error code
 
     actually_enabled = tuple(
         name
@@ -490,22 +537,30 @@ def handle_enable(options):
 
 
 def main():
-    """Console entry point for ``sopel-plugins``"""
+    """Console entry point for ``sopel-plugins``."""
     parser = build_parser()
     options = parser.parse_args()
     action = options.action
 
     if not action:
         parser.print_help()
-        return
+        return ERR_CODE
 
-    if action == 'list':
-        return handle_list(options)
-    elif action == 'show':
-        return handle_show(options)
-    elif action == 'configure':
-        return handle_configure(options)
-    elif action == 'disable':
-        return handle_disable(options)
-    elif action == 'enable':
-        return handle_enable(options)
+    try:
+        if action == 'list':
+            return handle_list(options)
+        elif action == 'show':
+            return handle_show(options)
+        elif action == 'configure':
+            return handle_configure(options)
+        elif action == 'disable':
+            return handle_disable(options)
+        elif action == 'enable':
+            return handle_enable(options)
+    except KeyboardInterrupt:
+        tools.stderr('Bye!')
+        return ERR_CODE
+    except config.ConfigurationNotFound as err:
+        tools.stderr(err)
+        tools.stderr('Use `sopel-config init` to create a new config file.')
+        return ERR_CODE
