@@ -28,7 +28,6 @@ import collections
 import datetime
 import functools
 import logging
-from random import randint
 import re
 import sys
 import time
@@ -45,8 +44,13 @@ if sys.version_info.major >= 3:
 
 LOGGER = logging.getLogger(__name__)
 
+CORE_QUERYTYPE = '999'
+"""WHOX querytype to indicate requests/responses from coretasks.
+
+Other plugins should use a different querytype.
+"""
+
 batched_caps = {}
-who_reqs = {}  # Keeps track of reqs coming from this plugin, rather than others
 
 
 def setup(bot):
@@ -483,11 +487,10 @@ def track_modes(bot, trigger):
 @module.unblockable
 def initial_modes(bot, trigger):
     """Populate channel modes from response to MODE request sent after JOIN."""
-    args = trigger.args[1:]
-    _parse_modes(bot, args)
+    _parse_modes(bot, trigger.args[1:], clear=True)
 
 
-def _parse_modes(bot, args):
+def _parse_modes(bot, args, clear=False):
     """Parse MODE message and apply changes to internal state."""
     channel_name = Identifier(args[0])
     if channel_name.is_nick():
@@ -496,10 +499,13 @@ def _parse_modes(bot, args):
         return
 
     channel = bot.channels[channel_name]
-    # Our old MODE parsing code checked for empty args. This would be a good
-    # place to re-implement that if necessary for a non-compliant IRCd, but for
-    # now just log malformed lines. After this we'll make a (possibly dangerous)
-    # assumption that the MODE message is more-or-less compliant.
+
+    # Unreal 3 sometimes sends an extraneous trailing space. If we're short an
+    # arg, we'll find out later.
+    if args[-1] == "":
+        args.pop()
+    # If any args are still empty, that's something we may not be prepared for,
+    # but let's continue anyway hoping they're trailing / not important.
     if len(args) < 2 or not all(args):
         LOGGER.debug(
             "The server sent a possibly malformed MODE message: %r", args)
@@ -517,6 +523,11 @@ def _parse_modes(bot, args):
         "Y": module.OPER,
     }
 
+    modes = {}
+    if not clear:
+        # Work on a copy for some thread safety
+        modes.update(channel.modes)
+
     # Process modes
     sign = ""
     param_idx = 0
@@ -529,33 +540,33 @@ def _parse_modes(bot, args):
 
         if char in chanmodes["A"]:
             # Type A (beI, etc) have a nick or address param to add/remove
-            if char not in channel.modes:
-                channel.modes[char] = set()
+            if char not in modes:
+                modes[char] = set()
             if sign == "+":
-                channel.modes[char].add(params[param_idx])
-            elif params[param_idx] in channel.modes[char]:
-                channel.modes[char].remove(params[param_idx])
+                modes[char].add(params[param_idx])
+            elif params[param_idx] in modes[char]:
+                modes[char].remove(params[param_idx])
             param_idx += 1
         elif char in chanmodes["B"]:
             # Type B (k, etc) always have a param
             if sign == "+":
-                channel.modes[char] = params[param_idx]
-            elif char in channel.modes:
-                channel.modes.pop(char)
+                modes[char] = params[param_idx]
+            elif char in modes:
+                modes.pop(char)
             param_idx += 1
         elif char in chanmodes["C"]:
             # Type C (l, etc) have a param only when setting
             if sign == "+":
-                channel.modes[char] = params[param_idx]
+                modes[char] = params[param_idx]
                 param_idx += 1
-            elif char in channel.modes:
-                channel.modes.pop(char)
+            elif char in modes:
+                modes.pop(char)
         elif char in chanmodes["D"]:
             # Type D (aciLmMnOpqrRst, etc) have no params
             if sign == "+":
-                channel.modes[char] = True
-            elif char in channel.modes:
-                channel.modes.pop(char)
+                modes[char] = True
+            elif char in modes:
+                modes.pop(char)
         elif char in mapping and (
             "PREFIX" not in bot.isupport or char in bot.isupport.PREFIX
         ):
@@ -599,6 +610,15 @@ def _parse_modes(bot, args):
             _send_who(bot, channel_name)
             return
 
+    if param_idx != len(params):
+        LOGGER.warning(
+            "Too many arguments received for MODE: args=%r chanmodes=%r",
+            args,
+            chanmodes,
+        )
+
+    channel.modes = modes
+
     LOGGER.info("Updated mode for channel: %s", channel.name)
     LOGGER.debug("Channel %r mode: %r", str(channel.name), channel.modes)
 
@@ -698,14 +718,12 @@ def _remove_from_channel(bot, nick, channel):
 def _send_who(bot, channel):
     if 'WHOX' in bot.isupport:
         # WHOX syntax, see http://faerion.sourceforge.net/doc/irc/whox.var
-        # Needed for accounts in WHO replies. The random integer is a param
-        # to identify the reply as one from this command, because if someone
-        # else sent it, we have no fucking way to know what the format is.
-        rand = str(randint(0, 999))
-        while rand in who_reqs:
-            rand = str(randint(0, 999))
-        who_reqs[rand] = channel
-        bot.write(['WHO', channel, 'a%nuachtf,' + rand])
+        # Needed for accounts in WHO replies. The `CORE_QUERYTYPE` parameter
+        # for WHO is used to identify the reply from the server and confirm
+        # that it has the requested format. WHO replies with different
+        # querytypes in the response were initiated elsewhere and will be
+        # ignored.
+        bot.write(['WHO', channel, 'a%nuachtf,' + CORE_QUERYTYPE])
     else:
         # We might be on an old network, but we still care about keeping our
         # user list updated
@@ -1243,8 +1261,9 @@ def account_notify(bot, trigger):
 @plugin.priority('medium')
 def recv_whox(bot, trigger):
     """Track ``WHO`` responses when ``WHOX`` is enabled."""
-    if len(trigger.args) < 2 or trigger.args[1] not in who_reqs:
+    if len(trigger.args) < 2 or trigger.args[1] != CORE_QUERYTYPE:
         # Ignored, some plugin probably called WHO
+        LOGGER.debug("Ignoring WHO reply for channel '%s'; not queried by coretasks", trigger.args[1])
         return
     if len(trigger.args) != 8:
         LOGGER.warning(
@@ -1307,16 +1326,6 @@ def recv_who(bot, trigger):
     _record_who(bot, channel, user, host, nick, away=away, modes=modes)
 
 
-@module.event(events.RPL_ENDOFWHO)
-@plugin.thread(False)
-@module.unblockable
-@plugin.priority('medium')
-def end_who(bot, trigger):
-    """Handle the end of a response to a ``WHO`` command (if needed)."""
-    if 'WHOX' in bot.isupport:
-        who_reqs.pop(trigger.args[1], None)
-
-
 @module.event('AWAY')
 @module.thread(False)
 @module.unblockable
