@@ -1,6 +1,6 @@
 # coding=utf-8
 """
-reddit.py - Sopel Reddit Module
+reddit.py - Sopel Reddit Plugin
 Copyright 2012, Elsie Powell, embolalia.com
 Copyright 2019, dgw, technobabbl.es
 Copyright 2019, deathbybandaid, deathbybandaid.net
@@ -8,19 +8,18 @@ Licensed under the Eiffel Forum License 2.
 
 https://sopel.chat
 """
-from __future__ import unicode_literals, absolute_import, print_function, division
+from __future__ import absolute_import, division, print_function, unicode_literals
 
 import datetime as dt
 import re
 import sys
-import textwrap
 
 import praw
 import prawcore
 import requests
 
+from sopel import plugin
 from sopel.formatting import bold, color, colors
-from sopel.module import commands, example, require_chanmsg, rule, url, NOLIMIT, OP
 from sopel.tools import time
 from sopel.tools.web import USER_AGENT
 
@@ -36,6 +35,7 @@ else:
     from HTMLParser import HTMLParser
     unescape = HTMLParser().unescape
 
+PLUGIN_OUTPUT_PREFIX = '[reddit] '
 
 domain = r'https?://(?:www\.|old\.|pay\.|ssl\.|[a-z]{2}\.)?reddit\.com'
 subreddit_url = r'%s/r/([\w-]+)/?$' % domain
@@ -45,6 +45,7 @@ user_url = r'%s/u(?:ser)?/([\w-]+)' % domain
 comment_url = r'%s/r/\S+?/comments/\S+?/\S+?/([\w-]+)' % domain
 image_url = r'https?://i\.redd\.it/\S+'
 video_url = r'https?://v\.redd\.it/([\w-]+)'
+gallery_url = r'https?://(?:www\.)?reddit\.com/gallery/([\w-]+)'
 
 
 def setup(bot):
@@ -91,7 +92,8 @@ def get_is_cakeday(entrytime):
     return is_cakeday
 
 
-@url(image_url)
+@plugin.url(image_url)
+@plugin.output_prefix(PLUGIN_OUTPUT_PREFIX)
 def image_info(bot, trigger, match):
     url = match.group(0)
     results = list(
@@ -103,11 +105,12 @@ def image_info(bot, trigger, match):
         oldest = results[-1]
     except IndexError:
         # Fail silently if the image link can't be mapped to a submission
-        return NOLIMIT
+        return plugin.NOLIMIT
     return say_post_info(bot, trigger, oldest.id, False, True)
 
 
-@url(video_url)
+@plugin.url(video_url)
+@plugin.output_prefix(PLUGIN_OUTPUT_PREFIX)
 def video_info(bot, trigger, match):
     # Get the video URL with a cheeky hack
     url = requests.head(
@@ -118,23 +121,32 @@ def video_info(bot, trigger, match):
             bot, trigger, re.match(post_url, url).group(1), False, True)
     except AttributeError:
         # Fail silently if we can't map the video link to a submission
-        return NOLIMIT
+        return plugin.NOLIMIT
 
 
-@url(post_url)
-@url(short_post_url)
+@plugin.url(post_url)
+@plugin.url(short_post_url)
+@plugin.output_prefix(PLUGIN_OUTPUT_PREFIX)
 def rpost_info(bot, trigger, match):
     match = match or trigger
     return say_post_info(bot, trigger, match.group(1))
 
 
+@plugin.url(gallery_url)
+@plugin.output_prefix(PLUGIN_OUTPUT_PREFIX)
+def rgallery_info(bot, trigger, match):
+    match = match or trigger
+    return say_post_info(bot, trigger, match.group(1), False)
+
+
 def say_post_info(bot, trigger, id_, show_link=True, show_comments_link=False):
     try:
         s = bot.memory['reddit_praw'].submission(id=id_)
 
-        message = ('[REDDIT] {title} {link}{nsfw} | {points} {points_text} '
-                   '({percent}) | {comments} comments | Posted by {author} | '
-                   'Created at {created}{comments_link}')
+        message = (
+            '{title} {link}{nsfw} | {points} {points_text} ({percent}) | '
+            '{comments} {comments_text} | Posted by {author} | '
+            'Created at {created}{comments_link}')
 
         subreddit = s.subreddit.display_name
         if not show_link:
@@ -180,7 +192,9 @@ def say_post_info(bot, trigger, id_, show_link=True, show_comments_link=False):
 
         points_text = 'point' if s.score == 1 else 'points'
 
-        percent = color(unicode(s.upvote_ratio * 100) + '%', point_color)
+        percent = color('{:.1%}'.format(s.upvote_ratio), point_color)
+
+        comments_text = 'comment' if s.num_comments == 1 else 'comments'
 
         comments_link = ''
         if show_comments_link:
@@ -193,25 +207,26 @@ def say_post_info(bot, trigger, id_, show_link=True, show_comments_link=False):
         title = unescape(s.title)
         message = message.format(
             title=title, link=link, nsfw=nsfw, points=s.score, points_text=points_text,
-            percent=percent, comments=s.num_comments, author=author, created=created,
-            comments_link=comments_link)
+            percent=percent, comments=s.num_comments, comments_text=comments_text,
+            author=author, created=created, comments_link=comments_link)
 
         bot.say(message)
     except prawcore.exceptions.NotFound:
-        bot.say('No such post.')
-        return NOLIMIT
+        bot.reply('No such post.')
+        return plugin.NOLIMIT
 
 
-@url(comment_url)
+@plugin.url(comment_url)
+@plugin.output_prefix(PLUGIN_OUTPUT_PREFIX)
 def comment_info(bot, trigger, match):
     """Shows information about the linked comment"""
     try:
         c = bot.memory['reddit_praw'].comment(match.group(1))
     except prawcore.exceptions.NotFound:
-        bot.say('No such comment.')
-        return NOLIMIT
+        bot.reply('No such comment.')
+        return plugin.NOLIMIT
 
-    message = ('[REDDIT] Comment by {author} | {points} {points_text} | '
+    message = ('Comment by {author} | {points} {points_text} | '
                'Posted at {posted} | {comment}')
 
     if c.author:
@@ -225,43 +240,57 @@ def comment_info(bot, trigger, match):
 
     # stolen from the function I (dgw) wrote for our github plugin
     lines = [line for line in c.body.splitlines() if line and line[0] != '>']
-    short = textwrap.wrap(lines[0], 250)[0]
-    if len(lines) > 1 or short != lines[0]:
-        short += ' […]'
 
     message = message.format(
         author=author, points=c.score, points_text=points_text,
-        posted=posted, comment=short)
+        posted=posted, comment=' '.join(lines))
 
-    bot.say(message)
+    bot.say(message, truncation=' […]')
 
 
 def subreddit_info(bot, trigger, match, commanded=False):
     """Shows information about the given subreddit"""
+    match_lower = match.lower()
+    if match_lower in ['all', 'popular']:
+        message = '{link}{nsfw} | {public_description}'
+        nsfw = ' ' + bold(color('[Possible NSFW]', colors.ORANGE))
+        link = "https://reddit.com/r/" + match_lower
+        public_description = ''
+        if match_lower == 'all':
+            public_description = ('Today\'s top content from hundreds of '
+                                  'thousands of Reddit communities.')
+        elif match_lower == 'popular':
+            public_description = ('The top trending content from some of '
+                                  'Reddit\'s most popular communities')
+        message = message.format(
+            link=link, nsfw=nsfw, public_description=public_description)
+        bot.say(message)
+        return plugin.NOLIMIT
+
     r = bot.memory['reddit_praw']
     try:
         r.subreddits.search_by_name(match, exact=True)
     except prawcore.exceptions.NotFound:
         if commanded:
-            bot.say('No such subreddit.')
+            bot.reply('No such subreddit.')
         # Fail silently if it wasn't an explicit command.
-        return NOLIMIT
+        return plugin.NOLIMIT
 
     try:
         s = r.subreddit(match)
         s.subreddit_type
     except prawcore.exceptions.Forbidden:
-        bot.say("r/" + match + " appears to be a private subreddit!")
-        return NOLIMIT
+        bot.reply("r/" + match + " appears to be a private subreddit!")
+        return plugin.NOLIMIT
     except prawcore.exceptions.NotFound:
-        bot.say("r/" + match + " appears to be a banned subreddit!")
-        return NOLIMIT
+        bot.reply("r/" + match + " appears to be a banned subreddit!")
+        return plugin.NOLIMIT
 
     link = "https://reddit.com/r/" + s.display_name
 
     created = get_time_created(bot, trigger, s.created_utc)
 
-    message = ('[REDDIT] {link}{nsfw} | {subscribers} subscribers | '
+    message = ('{link}{nsfw} | {subscribers} subscribers | '
                'Created at {created} | {public_description}')
 
     nsfw = ''
@@ -279,7 +308,8 @@ def subreddit_info(bot, trigger, match, commanded=False):
     message = message.format(
         link=link, nsfw=nsfw, subscribers='{:,}'.format(s.subscribers),
         created=created, public_description=s.public_description)
-    bot.say(message)
+
+    bot.say(message, truncation=' […]')
 
 
 def redditor_info(bot, trigger, match, commanded=False):
@@ -289,11 +319,11 @@ def redditor_info(bot, trigger, match, commanded=False):
         u.id  # shortcut to check if the user exists or not
     except prawcore.exceptions.NotFound:
         if commanded:
-            bot.say('No such Redditor.')
+            bot.reply('No such Redditor.')
         # Fail silently if it wasn't an explicit command.
-        return NOLIMIT
+        return plugin.NOLIMIT
 
-    message = '[REDDITOR] ' + u.name
+    message = u.name
     is_cakeday = get_is_cakeday(u.created_utc)
 
     if is_cakeday:
@@ -312,41 +342,43 @@ def redditor_info(bot, trigger, match, commanded=False):
     bot.say(message)
 
 
-@url(user_url)
+@plugin.url(user_url)
+@plugin.output_prefix(PLUGIN_OUTPUT_PREFIX)
 def auto_redditor_info(bot, trigger, match):
-    redditor_info(bot, trigger, match.group(1))
+    return redditor_info(bot, trigger, match.group(1))
 
 
-@url(subreddit_url)
+@plugin.url(subreddit_url)
+@plugin.output_prefix(PLUGIN_OUTPUT_PREFIX)
 def auto_subreddit_info(bot, trigger, match):
-    subreddit_info(bot, trigger, match.group(1))
+    return subreddit_info(bot, trigger, match.group(1))
 
 
-@require_chanmsg('.setsfw is only permitted in channels')
-@commands('setsafeforwork', 'setsfw')
-@example('.setsfw true')
-@example('.setsfw false')
+@plugin.require_chanmsg('Setting SFW status is only supported in a channel.')
+@plugin.require_privilege(plugin.OP)
+@plugin.command('setsafeforwork', 'setsfw')
+@plugin.example('.setsfw true')
+@plugin.example('.setsfw false')
+@plugin.output_prefix(PLUGIN_OUTPUT_PREFIX)
 def set_channel_sfw(bot, trigger):
     """
     Sets the Safe for Work status (true or false) for the current
     channel. Defaults to false.
     """
-    if bot.channels[trigger.sender].privileges[trigger.nick] < OP:
-        return
+    param = 'true'
+    if trigger.group(2) and trigger.group(3):
+        param = trigger.group(3).strip().lower()
+    sfw = param == 'true'
+    bot.db.set_channel_value(trigger.sender, 'sfw', sfw)
+    if sfw:
+        bot.say('%s is now flagged as SFW.' % trigger.sender)
     else:
-        param = 'true'
-        if trigger.group(2) and trigger.group(3):
-            param = trigger.group(3).strip().lower()
-        sfw = param == 'true'
-        bot.db.set_channel_value(trigger.sender, 'sfw', sfw)
-        if sfw:
-            bot.reply('Got it. %s is now flagged as SFW.' % trigger.sender)
-        else:
-            bot.reply('Got it. %s is now flagged as NSFW.' % trigger.sender)
+        bot.say('%s is now flagged as NSFW.' % trigger.sender)
 
 
-@commands('getsafeforwork', 'getsfw')
-@example('.getsfw [channel]')
+@plugin.command('getsafeforwork', 'getsfw')
+@plugin.example('.getsfw [channel]')
+@plugin.output_prefix(PLUGIN_OUTPUT_PREFIX)
 def get_channel_sfw(bot, trigger):
     """
     Gets the preferred channel's Safe for Work status, or the current
@@ -356,8 +388,9 @@ def get_channel_sfw(bot, trigger):
     if not channel:
         channel = trigger.sender
         if channel.is_nick():
-            return bot.say('{}getsfw with no channel param is only permitted in '
-                           'channels'.format(bot.config.core.help_prefix))
+            bot.reply('{}getsfw with no channel param is only permitted in '
+                      'channels.'.format(bot.config.core.help_prefix))
+            return
 
     channel = channel.strip()
 
@@ -368,31 +401,31 @@ def get_channel_sfw(bot, trigger):
         bot.say('%s is flagged as NSFW' % channel)
 
 
-@require_chanmsg('.setspoilfree is only permitted in channels')
-@commands('setspoilerfree', 'setspoilfree')
-@example('.setspoilfree true')
-@example('.setspoilfree false')
+@plugin.require_chanmsg('Only channels can be marked as spoiler-free.')
+@plugin.require_privilege(plugin.OP)
+@plugin.command('setspoilerfree', 'setspoilfree')
+@plugin.example('.setspoilfree true')
+@plugin.example('.setspoilfree false')
+@plugin.output_prefix(PLUGIN_OUTPUT_PREFIX)
 def set_channel_spoiler_free(bot, trigger):
     """
     Sets the Spoiler-Free status (true or false) for the current channel.
     Defaults to false.
     """
-    if bot.channels[trigger.sender].privileges[trigger.nick] < OP:
-        return
+    param = 'true'
+    if trigger.group(2) and trigger.group(3):
+        param = trigger.group(3).strip().lower()
+    spoiler_free = param == 'true'
+    bot.db.set_channel_value(trigger.sender, 'spoiler_free', spoiler_free)
+    if spoiler_free:
+        bot.say('%s is now flagged as spoiler-free.' % trigger.sender)
     else:
-        param = 'true'
-        if trigger.group(2) and trigger.group(3):
-            param = trigger.group(3).strip().lower()
-        spoiler_free = param == 'true'
-        bot.db.set_channel_value(trigger.sender, 'spoiler_free', spoiler_free)
-        if spoiler_free:
-            bot.reply('Got it. %s is now flagged as spoiler-free.' % trigger.sender)
-        else:
-            bot.reply('Got it. %s is now flagged as spoilers-allowed.' % trigger.sender)
+        bot.say('%s is now flagged as spoilers-allowed.' % trigger.sender)
 
 
-@commands('getspoilerfree', 'getspoilfree')
-@example('.getspoilfree [channel]')
+@plugin.command('getspoilerfree', 'getspoilfree')
+@plugin.example('.getspoilfree [channel]')
+@plugin.output_prefix(PLUGIN_OUTPUT_PREFIX)
 def get_channel_spoiler_free(bot, trigger):
     """
     Gets the channel's Spoiler-Free status, or the current channel's
@@ -402,8 +435,9 @@ def get_channel_spoiler_free(bot, trigger):
     if not channel:
         channel = trigger.sender
         if channel.is_nick():
-            return bot.say('{}getspoilfree with no channel param is only permitted '
-                           'in channels'.format(bot.config.core.help_prefix))
+            bot.reply('{}getspoilfree with no channel param is only permitted '
+                      'in channels.'.format(bot.config.core.help_prefix))
+            return
 
     channel = channel.strip()
 
@@ -414,34 +448,39 @@ def get_channel_spoiler_free(bot, trigger):
         bot.say('%s is flagged as spoilers-allowed' % channel)
 
 
-@rule(r'.*(?<!\S)/?(?P<prefix>r|u)/(?P<id>[a-zA-Z0-9-_]+)\b.*')
+@plugin.find(r'(?<!\S)/?(?P<prefix>r|u)/(?P<id>[a-zA-Z0-9-_]+)\b')
+@plugin.output_prefix(PLUGIN_OUTPUT_PREFIX)
 def reddit_slash_info(bot, trigger):
     searchtype = trigger.group('prefix').lower()
     match = trigger.group('id')
     if searchtype == "r":
-        return subreddit_info(bot, trigger, match, commanded=True)
+        return subreddit_info(bot, trigger, match, commanded=False)
     elif searchtype == "u":
-        return redditor_info(bot, trigger, match, commanded=True)
+        return redditor_info(bot, trigger, match, commanded=False)
 
 
-@commands('subreddit')
-@example('.subreddit plex')
+@plugin.command('subreddit')
+@plugin.example('.subreddit plex')
+@plugin.output_prefix(PLUGIN_OUTPUT_PREFIX)
 def subreddit_command(bot, trigger):
     # require input
     if not trigger.group(2):
-        return bot.reply('You must provide a subreddit name.')
+        bot.reply('You must provide a subreddit name.')
+        return
 
     # subreddit names do not contain spaces
     match = trigger.group(3)
     return subreddit_info(bot, trigger, match, commanded=True)
 
 
-@commands('redditor')
-@example('.redditor poem_for_your_sprog')
+@plugin.command('redditor')
+@plugin.example('.redditor poem_for_your_sprog')
+@plugin.output_prefix(PLUGIN_OUTPUT_PREFIX)
 def redditor_command(bot, trigger):
     # require input
     if not trigger.group(2):
-        return bot.reply('You must provide a Redditor name.')
+        bot.reply('You must provide a Redditor name.')
+        return
 
     # Redditor names do not contain spaces
     match = trigger.group(3)
