@@ -154,7 +154,7 @@ def f_remind(bot, trigger):
     if not os.path.exists(bot.tell_filename):
         return
 
-    if len(tellee) > 30:  # TODO: use server NICKLEN here when available
+    if len(tellee) > bot.isupport.get('NICKLEN', 30):
         bot.reply('That nickname is too long.')
         return
 
@@ -162,7 +162,7 @@ def f_remind(bot, trigger):
         tellee = tellee[1:]
 
     if tellee == bot.nick:
-        bot.reply("I'm here now; you can tell me whatever you want!")
+        bot.reply("I'm here now; you can %s me whatever you want!" % verb)
         return
 
     if tellee not in (tools.Identifier(teller), bot.nick, 'me'):
