@@ -3,7 +3,7 @@
 
 .. versionadded:: 7.0
 """
-from __future__ import unicode_literals, absolute_import, print_function, division
+from __future__ import absolute_import, division, print_function, unicode_literals
 
 
 def rawlist(*args):
